%===========================================================================================
% Genetics Algorithm for Travelling Salesperson Problem (TSP)
%  
% The aim of this function is investigating how important is crossover in TSP by
% comparing the results of GA 10 runs with crossover and 10 runs without crossover
% 
% 
% Updated on October 04, 2004 by Suyanto
%===========================================================================================

clc
clear all

XYcities      = [0 7; 0 19; 1 4; 1 11; 1 18; 2 2; 2 14; 3 10; 3 17; 4 0; ...
                 4 16; 5 13; 6 11; 7 5; 8 11; 8 19; 10 7; 11 1; 11 10; 12 18;...
                15 13; 15 16; 16 2; 16 17; 17 13; 17 19; 18 6; 18 10; 19 1; 20 15];

Ngenes        = length(XYcities(:,1));          % number of genes = number of cities

PopSize       = 100;
Pcrossover    = 0.8;
Pmutation     = 0.01;
MaxGeneration = 5000;

Fcutoff       = 1/110;                          % Cutoff fitness (got from TSPGAmain.m)
 
for Nrun=1:10,
  %--------------------------------------
  % Generate Population
  %--------------------------------------
  Population = TSPGenPop(PopSize,Ngenes);
  PopCross = Population;
  PopWithoutCross = Population;

  %--------------------------------------
  % Observation for GA with Crossover
  %--------------------------------------
  Population = PopCross;
  for generation=1:MaxGeneration,
    MaxFitness = TSPEvaluateIndividual(Population(1,:),Ngenes,XYcities);
    MinFitness = MaxFitness;
    for ii=1:PopSize,
      Fitness(ii) = TSPEvaluateIndividual(Population(ii,:),Ngenes,XYcities);
      if (Fitness(ii) >= MaxFitness),
         MaxFitness = Fitness(ii);
         IndBestIndividual = ii;
         BestRoute = Population(ii,:);
      end
      if (Fitness(ii) <= MinFitness),
         MinFitness = Fitness(ii);
      end
    end

    if MaxFitness > Fcutoff,
       break;
    end

    TempPopulation = Population;

    if mod(PopSize,2)==0,           % population size is even
       BeginIteration = 3;
       TempPopulation(1,:) = Population(IndBestIndividual,:);
       TempPopulation(2,:) = Population(IndBestIndividual,:);
    else                            % population size is odd
       BeginIteration = 2;
       TempPopulation(1,:) = Population(IndBestIndividual,:);
    end
       
    LinearFitness = LinearFitnessRanking(PopSize,Fitness,MaxFitness,MinFitness);

    for jj=BeginIteration:2:PopSize,
       IP1 = RWselection(PopSize,LinearFitness);
       IP2 = RWselection(PopSize,LinearFitness);
       if (rand < Pcrossover),
          Offspring = TSPCrossover(Population(IP1,:),Population(IP2,:),Ngenes);
          TempPopulation(jj,:) = Offspring(1,:);
          TempPopulation(jj+1,:) = Offspring(2,:);
        else
          TempPopulation(jj,:) = Population(IP1,:);
          TempPopulation(jj+1,:) = Population(IP2,:);
        end    
    end

    for kk=BeginIteration:PopSize,
      TempPopulation(kk,:) = TSPMutate(TempPopulation(kk,:),Ngenes,Pmutation);
    end   
  
    Population = TempPopulation;    % population replacement
      
  end

  ShortestPath1(Nrun) = 1/MaxFitness;
  GenerationData1(Nrun) = generation;  


  %--------------------------------------
  % Observation for GA without Crossover
  %--------------------------------------
  Population = PopWithoutCross;
  for generation=1:MaxGeneration,
    MaxFitness = TSPEvaluateIndividual(Population(1,:),Ngenes,XYcities);
    MinFitness = MaxFitness;
    for ii=1:PopSize,
      Fitness(ii) = TSPEvaluateIndividual(Population(ii,:),Ngenes,XYcities);
      if (Fitness(ii) >= MaxFitness),
         MaxFitness = Fitness(ii);
         IndBestIndividual = ii;
         BestRoute = Population(ii,:);
      end
      if (Fitness(ii) <= MinFitness),
        MinFitness = Fitness(ii);
      end
    end

    if MaxFitness > Fcutoff,
       break;
    end

    TempPopulation = Population;

    if mod(PopSize,2)==0,           % population size is even
       BeginIteration = 3;
       TempPopulation(1,:) = Population(IndBestIndividual,:);
       TempPopulation(2,:) = Population(IndBestIndividual,:);
    else                            % population size is odd
       BeginIteration = 2;
       TempPopulation(1,:) = Population(IndBestIndividual,:);
    end
       
    for kk=BeginIteration:PopSize,
      TempPopulation(kk,:) = TSPMutate(TempPopulation(kk,:),Ngenes,Pmutation);
    end   

    Population = TempPopulation;    % population replacement

  end

  ShortestPath2(Nrun) = 1/MaxFitness;
  GenerationData2(Nrun) = generation;  

  ObsData = [mean(ShortestPath1) std(ShortestPath1) mean(ShortestPath2) std(ShortestPath2)];
  save TSPtempresults.mat ObsData GenerationData1 GenerationData2
  
  Running = Nrun

  disp(['Comparing the results of GA 10 runs with crossover and 10 runs without']);
  disp(['                                                                      ']);
  disp(['----------------------------------------------------------------------']);
  disp(['  Average           Std. Dev.         Average             Std. Dev.   ']);
  disp(['Shortest Path    Shortest Path     Shortest Path        Shortest Path ']);
  disp(['With Crossover   With Crossover    Without Crossover    With Crossover']);
  disp(['----------------------------------------------------------------------']);
  disp(['  ', num2str(ObsData(1)),'          ', num2str(ObsData(2)), ... 
        '            ', num2str(ObsData(3)),'             ', num2str(ObsData(4))]);
  disp(['----------------------------------------------------------------------']);
  disp(['                                                                      ']);
  
end

save TSPfinalresults.mat ObsData GenerationData1 GenerationData2

disp(['Comparing the results of GA 10 runs with crossover and 10 runs without']);
disp(['                                                                      ']);
disp(['----------------------------------------------------------------------']);
disp(['  Average           Std. Dev.         Average             Std. Dev.   ']);
disp(['Shortest Path    Shortest Path     Shortest Path        Shortest Path ']);
disp(['With Crossover   With Crossover    Without Crossover    With Crossover']);
disp(['----------------------------------------------------------------------']);
disp(['  ', num2str(ObsData(1)),'          ', num2str(ObsData(2)), ... 
      '            ', num2str(ObsData(3)),'             ', num2str(ObsData(4))]);
disp(['----------------------------------------------------------------------']);