%===============================================================================
% Fungsi evolusi Satu Sub Populasi menggunakan Algoritma Genetika dengan 
% Binary Encoding. Di sini, evolusi hanya terjadi satu generasi.
% 
% Masukan
%   Populasi : populasi berisi sejumlah kromosom  
%   IM : matriks Input berukuran JPmasukan x JumMasukan
%   TM : matriks target berukuran JPmasukan x 1
%   Nbit : Jumlah bit yang mengkodekan satu variabel
%   Psilang : Probabilitas pindah silang
%   Pmutasi : Probabilitas mutasi
%
% Keluaran
%   PopulasiBaru: populasi baru hasil evolusi satu generasi
%
% Tanggal update 09 Januari 2005
% Suyanto - Teknik Informatika - Sekolah Tinggi Teknologi Telkom 
% Jl Telekomunikasi No 1 Dayeuh Kolot - Bandung 40257
%===============================================================================

function PopulasiBaru = EvolusiSatuSubPopulasi(Populasi,IM,TM,Nbit,Psilang,Pmutasi)

JumMasukan  = 3;                     % Jumlah masukan
JPmasukan   = 2^JumMasukan;          % Jumlah pola masukan
JumGen      = Nbit*(JumMasukan+1)^2; % Jumlah gen
Nvar        = JumGen/Nbit;           % Jumlah variabel
Rb          = -10;                   % Batas bawah dari rentang yang ditentukan  
Ra          = 10;                    % Batas atas dari rentang yang ditentukan

UkPop       = length(Populasi(:,1)); % Jumlah kromosom dalam satu sub populasi

FFNNstruk = DekodekanKromosom(Populasi(1,:),Nvar,Nbit,Ra,Rb);
Fitness(1) = BinaryEvalInd(FFNNstruk,JumMasukan,JPmasukan,IM,TM);
MaxF = Fitness(1);
MinF = Fitness(1);
IndeksIndividuTerbaik = 1;
for ii=1:UkPop,
    FFNNstruk = DekodekanKromosom(Populasi(ii,:),Nvar,Nbit,Ra,Rb);
    Fitness(ii) = BinaryEvalInd(FFNNstruk,JumMasukan,JPmasukan,IM,TM);
    if (Fitness(ii) > MaxF),
        MaxF = Fitness(ii);
        IndeksIndividuTerbaik = ii;
        FFNNterbaik = FFNNstruk;
    end
    if (Fitness(ii) <= MinF),
        MinF = Fitness(ii);
    end
end

FitnessRataRata = mean(Fitness);
   
TemPopulasi = Populasi;

% Elitisme:
% - Buat satu kopi kromosom terbaik jika ukuran populasi ganjil
% - Buat dua kopi kromosom terbaik jika ukuran populasi genap
if mod(UkPop,2)==0,           % ukuran populasi genap
    IterasiMulai = 3;
    TemPopulasi(1,:) = Populasi(IndeksIndividuTerbaik,:);
    TemPopulasi(2,:) = Populasi(IndeksIndividuTerbaik,:);
else                          % ukuran populasi ganjil
    IterasiMulai = 2;
    TemPopulasi(1,:) = Populasi(IndeksIndividuTerbaik,:);
end
       
LinearFitness = LinearFitnessRanking(UkPop,Fitness,MaxF,MinF);

for jj=IterasiMulai:2:UkPop,
    IP1 = RouletteWheel(UkPop,LinearFitness);
    IP2 = RouletteWheel(UkPop,LinearFitness);
    if (rand < Psilang),
        Anak = PindahSilang(Populasi(IP1,:),Populasi(IP2,:),JumGen);
        TemPopulasi(jj,:) = Anak(1,:);
        TemPopulasi(jj+1,:) = Anak(2,:);
    else
        TemPopulasi(jj,:) = Populasi(IP1,:);
        TemPopulasi(jj+1,:) = Populasi(IP2,:);
    end    
end

for kk=IterasiMulai:UkPop,
    TemPopulasi(kk,:) = Mutasi(TemPopulasi(kk,:),JumGen,Pmutasi);
end   
   
PopulasiBaru = TemPopulasi;