%===============================================================================
% Algoritma Genetika berbasis sub populasi untuk melatih FFNN
% 
% Memanggil fungsi-fungsi berikut
% 1. BangMatrixIT
% 2. InisialisasiPopulasi
% 3. DekodekanKromosom
% 4. BinaryEvalInd
% 5. EvolusiSatuSubPopulasi
%
% Tanggal update 09 Januari 2005
% Suyanto - Teknik Informatika - Sekolah Tinggi Teknologi Telkom 
% Jl Telekomunikasi No 1 Dayeuh Kolot - Bandung 40257
%===============================================================================

clc
clear all

JumMasukan  = 3;                     % Jumlah masukan
JPmasukan   = 2^JumMasukan;          % Jumlah pola masukan
Nbit        = 20;                    % Jumlah bit yang mengkodekan satu variabel 
JumGen      = Nbit*(JumMasukan+1)^2; % Jumlah gen
Nvar        = JumGen/Nbit;           % Jumlah variabel
Rb          = -10;                   % Batas bawah interval  
Ra          = 10;                    % Batas atas interval
MinDelta    = 0.01;                  % Delta minimum yang diharapkan
Fthreshold  = 1/MinDelta;            % Threshold untuk nilai Fitness
Bgraf       = Fthreshold;            % Untuk menangani tampilan grafis

UkPop       = 100;                   % Jumlah kromosom dalam populasi     
Psilang     = 0.8;                   % Probabilitas pindah silang
Pmutasi     = 0.03;                  % Probabilitas mutasi
MaxG        = 2000;                  % Jumlah generasi        

JumSubPop    = 4;                    % jumlah sub populasi
JumIndSubPop = fix(UkPop/JumSubPop); % jumlah individu dalam satu sub populasi
ProbTunel    = 0.001;                % probabilitas tunelling 

% Inisialisasi grafis
hfig = figure;
hold on
title('Algoritma Genetika dengan Sub Populasi untuk pelatihan FFNN')
set(hfig, 'position', [50,50,600,400]);
set(hfig, 'DoubleBuffer', 'on');
axis([1 MaxG 0 Bgraf]);
hbestplot1 = plot(1:MaxG,zeros(1,MaxG));
hbestplot2 = plot(1:MaxG,zeros(1,MaxG));
htext1 = text(0.6*MaxG,0.25*Bgraf,sprintf('Fitness terbaik: %7.6f', 0.0));
htext2 = text(0.6*MaxG,0.20*Bgraf,sprintf('Fitness rata-rata: %7.6f', 0.0));
htext3 = text(0.6*MaxG,0.15*Bgraf,sprintf('Ukuran populasi: %3.0f', 0.0));
htext4 = text(0.6*MaxG,0.10*Bgraf,sprintf('Prob. Pindah Silang: %4.3f', 0.0));
htext5 = text(0.6*MaxG,0.05*Bgraf,sprintf('Prob. Mutasi: %4.3f', 0.0));
xlabel('Generasi');
ylabel('Fitness');
hold off
drawnow;

% Bangkitkan matrix Input and Target untuk pelatihan FFNN
[IM,TM] = BangMatrixIT(JumMasukan,JPmasukan);

% Inisialisasi Populasi 
Populasi = InisialisasiPopulasi(UkPop,JumGen);

for generasi=1:MaxG,
   %-------------------------
   % Evaluasi populasi global 
   %-------------------------
   FFNNstruk = DekodekanKromosom(Populasi(1,:),Nvar,Nbit,Ra,Rb);
   Fitness(1) = BinaryEvalInd(FFNNstruk,JumMasukan,JPmasukan,IM,TM);
   MaxF = Fitness(1);
   IndeksIndividuTerbaik = 1;
   for ii=2:UkPop,
     FFNNstruk = DekodekanKromosom(Populasi(ii,:),Nvar,Nbit,Ra,Rb);
     Fitness(ii) = BinaryEvalInd(FFNNstruk,JumMasukan,JPmasukan,IM,TM);
     if (Fitness(ii) > MaxF),
        MaxF = Fitness(ii);
        IndeksIndividuTerbaik = ii;
        FFNNterbaik = FFNNstruk;
     end
   end

   FitnessRataRata = mean(Fitness);
   
   plotvector1 = get(hbestplot1,'YData');
   plotvector1(generasi) = MaxF;
   set(hbestplot1,'YData',plotvector1);
   plotvector2 = get(hbestplot2,'YData');
   plotvector2(generasi) = FitnessRataRata;
   set(hbestplot2,'YData',plotvector2);
   set(htext1,'String',sprintf('Fitness terbaik: %7.6f', MaxF));
   set(htext2,'String',sprintf('Fitness rata-rata: %7.6f', FitnessRataRata));
   set(htext3,'String',sprintf('Ukuran populasi: %3.0f', UkPop));
   set(htext4,'String',sprintf('Prob. Pindah Silang: %4.3f', Psilang));
   set(htext5,'String',sprintf('Probabilitas Mutasi: %4.3f', Pmutasi));
   drawnow

   if MaxF > Fthreshold,
      break;
   end

    %---------------------------------------------------------
    % Evaluasi sub populasi ke-1 sampai JumSubPop-1
    %---------------------------------------------------------
    for ISP=1:JumSubPop-1,
        SubPop = Populasi((ISP-1)*JumIndSubPop+1:ISP*JumIndSubPop,:);
        SubPopBaru = EvolusiSatuSubPopulasi(SubPop,IM,TM,Nbit,Psilang,Pmutasi);
        Populasi((ISP-1)*JumIndSubPop+1:ISP*JumIndSubPop,:) = SubPopBaru;
    end
    
    %---------------------------------------------------------
    % Evaluasi sub populasi terakhir karena jumlah kromosomnya 
    % bisa berbeda dengan sub populasi yang lain
    %---------------------------------------------------------
    SubPop = Populasi((JumSubPop-1)*JumIndSubPop+1:UkPop,:);
    SubPopBaru = EvolusiSatuSubPopulasi(SubPop,IM,TM,Nbit,Psilang,Pmutasi);
    Populasi((JumSubPop-1)*JumIndSubPop+1:UkPop,:) = SubPopBaru;
    
    % Tunneling antar sub populasi
    for ISP=1:JumSubPop-1,
        for Itt=ISP+1:JumSubPop,
            if rand < ProbTunel
                % Pilih satu kromosom pada satu sub populasi
                ind1 = fix(1+rand*JumIndSubPop);        
                if Itt==JumSubPop,
                    % Pilih satu kromosom pada sub poppulasi yang lain
                    ind2 = fix(1+rand*(UkPop-(JumSubPop-1)*JumIndSubPop));
                else
                    % Pilih satu kromosom pada sub poppulasi yang lain
                    ind2 = fix(1+rand*JumIndSubPop);
                end
                KromTemp = Populasi((ISP-1)*JumIndSubPop+ind1,:);
                Populasi((ISP-1)*JumIndSubPop+ind1,:) = Populasi((Itt-1)*JumIndSubPop+ind2,:);
                Populasi((Itt-1)*JumIndSubPop+ind2,:) = KromTemp;
            end
        end
    end
end

FFNNterbaik
save FFNNterbaik.mat FFNNterbaik