%===============================================================================
% Algoritma Genetika berbasis grid untuk melatih FFNN
% 
% Memanggil fungsi-fungsi berikut
% 1. BangMatrixIT
% 2. InisialisasiPopulasi
% 3. DekodekanKromosom
% 4. BinaryEvalInd
% 5. GBTournamentSelection
% 6. GBPindahSilang
% 7. Mutasi
% 8. GBReplacement
%
% Tanggal update 09 Januari 2005
% Suyanto - Teknik Informatika - Sekolah Tinggi Teknologi Telkom 
% Jl Telekomunikasi No 1 Dayeuh Kolot - Bandung 40257
%===============================================================================

clc
clear all

Jbar       = 10;                     % Jumlah baris
Jkol       = 10;                     % Jumlah kolom
UkPop      = Jbar * Jkol;            % Ukuran populasi (ukuran grid)
JLtetangga = 1;                      % Jumlah lapisan ketetanggaan

JumMasukan  = 3;                     % Jumlah masukan
JPmasukan   = 2^JumMasukan;          % Jumlah pola masukan
Nbit        = 20;                    % Jumlah bit yang mengkodekan satu variabel 
JumGen      = Nbit*(JumMasukan+1)^2; % Jumlah gen
Nvar        = JumGen/Nbit;           % Jumlah variabel
Rb          = -10;                   % Batas bawah interval  
Ra          = 10;                    % Batas atas interval
MinDelta    = 0.01;                  % Delta minimum yang diharapkan
Fthreshold  = 1/MinDelta;            % Threshold untuk nilai Fitness
Bgraf       = Fthreshold;            % Untuk menangani tampilan grafis

Psilang     = 0.8;                   % Probabilitas pindah silang
Pmutasi     = 0.03;                  % Probabilitas mutasi
UkTour      = 4;                     % Ukuran tournament
ProbTour    = 0.8;                   % Probabilitas tournament
MaxG        = 2000;                  % Jumlah generasi        


% Inisialisasi grafis
hfig = figure;
hold on
title('Algoritma Genetika berbasis Grid untuk pelatihan FFNN')
set(hfig, 'position', [50,50,600,400]);
set(hfig, 'DoubleBuffer', 'on');
axis([1 MaxG 0 Bgraf]);
hbestplot1 = plot(1:MaxG,zeros(1,MaxG));
hbestplot2 = plot(1:MaxG,zeros(1,MaxG));
htext1 = text(0.6*MaxG,0.35*Bgraf,sprintf('Fitness terbaik: %7.6f', 0.0));
htext2 = text(0.6*MaxG,0.30*Bgraf,sprintf('Fitness rata-rata: %7.6f', 0.0));
htext3 = text(0.6*MaxG,0.25*Bgraf,sprintf('Ukuran populasi: %3.0f', 0.0));
htext4 = text(0.6*MaxG,0.20*Bgraf,sprintf('Prob. Pindah Silang: %4.3f', 0.0));
htext5 = text(0.6*MaxG,0.15*Bgraf,sprintf('Prob. Mutasi: %4.3f', 0.0));
htext6 = text(0.6*MaxG,0.10*Bgraf,sprintf('Ukuran Tournament: %2.0f', 0.0));
htext7 = text(0.6*MaxG,0.05*Bgraf,sprintf('Prob. Tournament: %4.3f', 0.0));

xlabel('Generasi');
ylabel('Fitness');
hold off
drawnow;

% Bangkitkan matrix Input and Target untuk pelatihan FFNN
[IM,TM] = BangMatrixIT(JumMasukan,JPmasukan);

% Inisialisasi Populasi 
Populasi = InisialisasiPopulasi(UkPop,JumGen);

% Membangun Toroidal Space yang berisi indeks kromosom pada populasi
TSPI = [Jbar*Jkol (Jbar-1)*Jkol+1:Jbar*Jkol (Jbar-1)*Jkol+1]; % baris pertama
for ii=1:Jbar,
    % baris tengah
    TSPI = [TSPI ; Jkol*ii (ii-1)*Jkol+1:Jkol*ii (ii-1)*Jkol+1];
end
TSPI = [TSPI ; Jkol 1:Jkol 1]; % baris teakhir

for generasi=1:MaxG,
   FFNNstruk = DekodekanKromosom(Populasi(1,:),Nvar,Nbit,Ra,Rb);
   Fitness(1) = BinaryEvalInd(FFNNstruk,JumMasukan,JPmasukan,IM,TM);
   MaxF = Fitness(1);
   IndeksIndividuTerbaik = 1;
   for ii=2:UkPop,
     FFNNstruk = DekodekanKromosom(Populasi(ii,:),Nvar,Nbit,Ra,Rb);
     Fitness(ii) = BinaryEvalInd(FFNNstruk,JumMasukan,JPmasukan,IM,TM);
     if (Fitness(ii) > MaxF),
        MaxF = Fitness(ii);
        IndeksIndividuTerbaik = ii;
        FFNNterbaik = FFNNstruk;
     end
   end

   FitnessRataRata = mean(Fitness);
   
   plotvector1 = get(hbestplot1,'YData');
   plotvector1(generasi) = MaxF;
   set(hbestplot1,'YData',plotvector1);
   plotvector2 = get(hbestplot2,'YData');
   plotvector2(generasi) = FitnessRataRata;
   set(hbestplot2,'YData',plotvector2);
   set(htext1,'String',sprintf('Fitness terbaik: %7.6f', MaxF));
   set(htext2,'String',sprintf('Fitness rata-rata: %7.6f', FitnessRataRata));
   set(htext3,'String',sprintf('Ukuran populasi: %3.0f', UkPop));
   set(htext4,'String',sprintf('Prob. Pindah Silang: %4.3f', Psilang));
   set(htext5,'String',sprintf('Prob. Mutasi: %4.3f', Pmutasi));
   set(htext6,'String',sprintf('Ukuran Tournament: %2.0f', UkTour));
   set(htext7,'String',sprintf('Prob. Tournament: %4.3f', ProbTour));
   drawnow
   
   if MaxF > Fthreshold,
      break;
   end
   
   TemPop = Populasi;

   if mod(UkPop,2)==0,           % ukuran populasi genap
      IterasiMulai = 3;
   else                          % ukuran populasi ganjil
      IterasiMulai = 2;
   end
       
   for jj=IterasiMulai:UkPop,
     IP1 = GBTourSelection(UkPop,Fitness,UkTour,ProbTour);
     if (rand < Psilang),
       [Anak,IP2] = GBPindahSilang(TemPop,JumGen,IP1,TSPI,JLtetangga,Jbar,Jkol);
       MAnak1 = Mutasi(Anak(1,:),JumGen,Pmutasi);
       MAnak2 = Mutasi(Anak(2,:),JumGen,Pmutasi);
       x1 = DekodekanKromosom(MAnak1,Nvar,Nbit,Ra,Rb);
       x2 = DekodekanKromosom(MAnak2,Nvar,Nbit,Ra,Rb);
       FMA1 = BinaryEvalInd(x1,JumMasukan,JPmasukan,IM,TM);
       FMA2 = BinaryEvalInd(x2,JumMasukan,JPmasukan,IM,TM);
       PopBaru = GBReplacement(TemPop,Fitness,IP1,IP2,MAnak1,MAnak2,FMA1,FMA2);
       TemPop = PopBaru;
     else
       MutKrom = Mutasi(TemPop(IP1,:),JumGen,Pmutasi);
       x1 = DekodekanKromosom(MutKrom,Nvar,Nbit,Ra,Rb);
       FitnessMutKrom = BinaryEvalInd(x1,JumMasukan,JPmasukan,IM,TM);
       if FitnessMutKrom > Fitness(IP1),
          TemPop(IP1,:) = MutKrom;
       end
     end
   end
  
   % Elitisme dilakukan setelah operator-operator Grid-based dilakukan
   % untuk mempertahankan individu terbaik
   % - Buat satu kopi kromosom terbaik jika ukuran populasi ganjil
   % - Buat dua kopi kromosom terbaik jika ukuran populasi genap
   if mod(UkPop,2)==0,
      TemPop(1,:) = Populasi(IndeksIndividuTerbaik,:);
      TemPop(2,:) = Populasi(IndeksIndividuTerbaik,:);
   else
      TemPop(1,:) = Populasi(IndeksIndividuTerbaik,:);
   end
       
   Populasi = TemPop;    % generational replacement
   
end

FFNNterbaik
save FFNNterbaik.mat FFNNterbaik