%===============================================================================
% Grid-based Crossover
% Memindah-silangkan bagian kromosom Bapak dan Ibu yang dipotong secara random, 
% sehingga dihasilkan dua buah kromosom baru yang disebut Anak. 
% Kromosom Ibu dipilih dari kromosom-kromosom tetangga pada Toroidal Space
%
% Masukan 
%   Pop : Populasi
%   JGen : jumlah gen
%   IP1  : Indeks orang tua 1
%   TPSI  : Toroidal Space Population Indices
%   JLtetangga: Jumlah lapisan ketetanggaan
%   Jbar : Jumlah baris pada Toroidal Space
%   Jkol : jumlah kolom pada Toroidal Space
%   
% Keluaran 
%   Anak : kromosom hasil pindah silang, matriks berukuran 1 x JGen
%   IP2  : Indeks orang tua 2
%
% Tanggal update 28 Desember 2004
% Suyanto - Teknik Informatika - Sekolah Tinggi Teknologi Telkom 
% Jl Telekomunikasi No 1 Dayeuh Kolot - Bandung 40257
%===============================================================================

function [Anak,IP2] = GBPindahSilang(Pop,JGen,IP1,TSPI,JLtetangga,Jbar,Jkol)

% Indeks bapak pada baris Toroidal Space
Bbaris = fix((IP1+Jkol-1)/Jkol) + JLtetangga;
ModKolom = mod(IP1,Jkol);
if ModKolom == 0,         % jika kolom sudah maksimum
    ModKolom = Jkol;                              
end
% Indeks bapak pada kolom Toroidal Space
Bkolom = ModKolom + JLtetangga;

% Bangkitkan indeks tetangga sebagai kandidat ibu
IndKandidatIbu = [];
for ii=Bbaris-JLtetangga:Bbaris+JLtetangga,
    for jj=Bkolom-JLtetangga:Bkolom+JLtetangga,
        if (ii ~= Bbaris) | (jj ~= Bkolom),
            IndKandidatIbu = [IndKandidatIbu TSPI(ii,jj)];
        end
    end
end

% Pilih salah satu dari IndKandidatIbu sebagai indeks ibu
ind = 1 + fix(rand*length(IndKandidatIbu));
IP2 = IndKandidatIbu(ind);   % indeks ibu

Ibu = Pop(IP2,:);      % kromosom ibu
Bapak = Pop(IP1,:);    % kromosom bapak

TP = 1 + fix(rand*(JGen-1));  % random titik potong

Anak(1,:) = [Bapak(1:TP) Ibu(TP+1:JGen)];
Anak(2,:) = [Ibu(1:TP) Bapak(TP+1:JGen)];
