%===============================================================================
% Algoritma Genetika dengan Binary Encoding untuk melatih FFNN
% 
% Memanggil fungsi-fungsi berikut
% 1. BangMatrixIT
% 2. GEInisialisasiPopulasi
% 3. GEDekode
% 4. GEEvaluasiIndividu
% 5. LinearFitnessRanking
% 6. RouletteWheel
% 7. PindahSilang
% 8. Mutasi
%
% Tanggal update 11 Januari 2005
% Suyanto - Teknik Informatika - Sekolah Tinggi Teknologi Telkom 
% Jl Telekomunikasi No 1 Dayeuh Kolot - Bandung 40257
%===============================================================================

clc
clear all

HHbesar = ['ABCD'];              % Himpunan huruf besar
HHkecil = ['abcdefghijklmnop'];  % Himpunan huruf kecil
JHB     = length(HHbesar);       % Jumlah huruf besar dalam himpunan
JHK     = length(HHkecil);       % Jumlah huruf kecil dalam himpunan

PBK     = 4;    % Jumlah huruf besar dalam satu grup setelah simbol start S
PKK     = 4;    % Jumlah huruf kecil dalam satu grup setelah huruf besar

JumBesarKecil = 20; % Jumlah grup dalam satu kromosom
JumGen = 1 + PBK + JumBesarKecil * (PKK+1);  % Jumlah gen 
BilKecil   = 10^-1;          % untuk menghindari pembagian dengan 0
Fthreshold = 1/BilKecil;     % Threshold untuk nilai Fitness
Bgraf      = Fthreshold;     % Untuk menangani tampilan grafis

UkPop      = 100;            % Jumlah kromosom dalam populasi     
Psilang    = 0.8;            % Probabilitas pindah silang
Pmutasi    = 0.01;           % Probabilitas mutasi
MaxG       = 500;            % Jumlah generasi        

% Inisialisasi grafis
hfig = figure;
hold on
title('AG dengan gramatical encoding untuk membangun struktur Neural Network')
set(hfig, 'position', [50,50,600,400]);
set(hfig, 'DoubleBuffer', 'on');
axis([1 MaxG 0 Bgraf]);
hbestplot1 = plot(1:MaxG,zeros(1,MaxG));
hbestplot2 = plot(1:MaxG,zeros(1,MaxG));
htext1 = text(0.6*MaxG,0.25*Bgraf,sprintf('Fitness terbaik: %7.6f', 0.0));
htext2 = text(0.6*MaxG,0.20*Bgraf,sprintf('Fitness rata-rata: %7.6f', 0.0));
htext3 = text(0.6*MaxG,0.15*Bgraf,sprintf('Ukuran populasi: %3.0f', 0.0));
htext4 = text(0.6*MaxG,0.10*Bgraf,sprintf('Prob. Pindah Silang: %4.3f', 0.0));
htext5 = text(0.6*MaxG,0.05*Bgraf,sprintf('Prob. Mutasi: %4.3f', 0.0));
xlabel('Generasi');
ylabel('Fitness');
hold off
drawnow;

% Bangkitkan NN target
NNtarget = BangkitkanNNtarget;

% Bangkitkan Matriks 2 x 2 untuk 16 simbol huruf kecil
[a,b,c,d,e,f,g,h,i,j,k,l,m,n,o,p] = BangkitkanMatriks16simbol;

% Inisialisasi Populasi 
Populasi = GEInisialisasiPopulasi(UkPop,HHbesar,HHkecil,JHB,JHK,PBK,PKK,JumGen);

for generasi=1:MaxG,
    NNstruk = GEDekode(Populasi(1,:),JumGen,PBK,PKK,a,b,c,d,e,f,g,h,i,j,k,l,m,n,o,p);
    Fitness(1) = GEEvaluasiIndividu(NNstruk,NNtarget,BilKecil);
    MaxF = Fitness(1);
    MinF = Fitness(1);
    IndeksIndividuTerbaik = 1;
    for ii=2:UkPop,
        NNstruk = GEDekode(Populasi(ii,:),JumGen,PBK,PKK,a,b,c,d,e,f,g,h,i,j,k,l,m,n,o,p);
        Fitness(ii) = GEEvaluasiIndividu(NNstruk,NNtarget,BilKecil);
        if (Fitness(ii) > MaxF),
            MaxF = Fitness(ii);
            IndeksIndividuTerbaik = ii;
            KromosomTerbaik = Populasi(ii,:);
            NNstrukTerbaik = NNstruk;
        end
        if (Fitness(ii) <= MinF),
           MinF = Fitness(ii);
        end
    end

   FitnessRataRata = mean(Fitness);
   
   plotvector1 = get(hbestplot1,'YData');
   plotvector1(generasi) = MaxF;
   set(hbestplot1,'YData',plotvector1);
   plotvector2 = get(hbestplot2,'YData');
   plotvector2(generasi) = FitnessRataRata;
   set(hbestplot2,'YData',plotvector2);
   set(htext1,'String',sprintf('Fitness terbaik: %7.6f', MaxF));
   set(htext2,'String',sprintf('Fitness rata-rata: %7.6f', FitnessRataRata));
   set(htext3,'String',sprintf('Ukuran populasi: %3.0f', UkPop));
   set(htext4,'String',sprintf('Prob. Pindah Silang: %4.3f', Psilang));
   set(htext5,'String',sprintf('Probabilitas Mutasi: %4.3f', Pmutasi));
   drawnow

   if MaxF >= Fthreshold,
      break;
   end

   TemPopulasi = Populasi;

   % Elitisme:
   % - Buat satu kopi kromosom terbaik jika ukuran populasi ganjil
   % - Buat dua kopi kromosom terbaik jika ukuran populasi genap
   if mod(UkPop,2)==0,           % ukuran populasi genap
      IterasiMulai = 3;
      TemPopulasi(1,:) = Populasi(IndeksIndividuTerbaik,:);
      TemPopulasi(2,:) = Populasi(IndeksIndividuTerbaik,:);
   else                          % ukuran populasi ganjil
      IterasiMulai = 2;
      TemPopulasi(1,:) = Populasi(IndeksIndividuTerbaik,:);
   end
       
   LinearFitness = LinearFitnessRanking(UkPop,Fitness,MaxF,MinF);

   for jj=IterasiMulai:2:UkPop,
       IP1 = RouletteWheel(UkPop,LinearFitness);
       IP2 = RouletteWheel(UkPop,LinearFitness);
       if (rand < Psilang),
          Anak = PindahSilang(Populasi(IP1,:),Populasi(IP2,:),JumGen);
          TemPopulasi(jj,:) = Anak(1,:);
          TemPopulasi(jj+1,:) = Anak(2,:);
       else
          TemPopulasi(jj,:) = Populasi(IP1,:);
          TemPopulasi(jj+1,:) = Populasi(IP2,:);
       end    
   end

   for kk=IterasiMulai:UkPop,
      TemPopulasi(kk,:) = GEMutasi(TemPopulasi(kk,:),JumGen,HHbesar,HHkecil,JHB,JHK,PBK,Pmutasi);
   end   
   
   Populasi = TemPopulasi;
   
end

KromosomTerbaik
NNstrukTerbaik
save NNstrukTerbaik.mat NNstrukTerbaik